import sqlite3
from datetime import datetime

_enable_foreign_keys = "PRAGMA foreign_keys = ON;"

_create_table_orders = """CREATE TABLE IF NOT EXISTS orders (
    gamekey TEXT,
    machine_name TEXT,
    human_name TEXT,
    category TEXT,
    total REAL,
    created TEXT,
    url TEXT,
    PRIMARY KEY (gamekey)
);
"""

_create_table_products = """CREATE TABLE IF NOT EXISTS products (
    machine_name TEXT,
    human_name TEXT,
    icon TEXT,
    url TEXT,
    editor_human_name TEXT,
    editor_machine_name TEXT,
    orders_gamekey TEXT,
    FOREIGN KEY (orders_gamekey) REFERENCES orders(gamekey),
    PRIMARY KEY (machine_name)
);
"""

_create_table_downloads = """CREATE TABLE IF NOT EXISTS downloads (
    name TEXT,
    platform TEXT,
    machine_name TEXT,
    title TEXT,
    filename TEXT,
    human_size TEXT,
    size INTEGER,
    sha1 TEXT,
    md5 TEXT,
    first_seen TEXT,
    products_machine_name TEXT,
    FOREIGN KEY (products_machine_name) REFERENCES products(machine_name),
    PRIMARY KEY (products_machine_name, filename, size, sha1, md5)
);
"""

_insert_orders = """INSERT OR IGNORE INTO orders VALUES (?,?,?,?,?,?,?);"""
_insert_products = """INSERT OR IGNORE INTO products VALUES (?,?,?,?,?,?,?);"""
_insert_downloads = """INSERT OR IGNORE INTO downloads VALUES (?,?,?,?,?,?,?,?,?,?,?);"""

_select_orders = """SELECT * FROM orders ORDER BY %s;"""
_select_orders_like = """SELECT * FROM orders WHERE human_name GLOB ? ORDER BY %s;"""

_select_products = """SELECT * FROM products ORDER BY %s;"""

_select_downloads = """SELECT max(datetime(first_seen)),
orders.machine_name AS order_machine_name,
products.machine_name AS product_machine_name,
downloads.machine_name AS machine_name,
*
FROM downloads,products,orders
WHERE downloads.products_machine_name = products.machine_name
AND products.orders_gamekey = orders.gamekey
%s
GROUP BY name;
"""

_find_filename = """SELECT products.machine_name, products.orders_gamekey AS gamekey
FROM downloads,products
WHERE downloads.products_machine_name = products.machine_name AND downloads.filename = ?
ORDER BY datetime(downloads.first_seen) DESC
LIMIT 1;
"""

def order2tuple(order):
    return (
        order["gamekey"],
        order["machine_name"],
        order["human_name"],
        order["category"],
        float(order["total"]),
        order["created"],
        order["url"],
    )

def product2tuple(product, order):
    return (
        product["machine_name"],
        product["human_name"],
        product["icon"],
        product["url"],
        product["editor_human_name"],
        product["editor_machine_name"],
        order,
    )

def download2tuple(download, product):
    return (
        download["name"],
        download["platform"],
        download["machine_name"],
        download["title"],
        download["filename"],
        download["human_size"],
        download["size"],
        download["sha1"],
        download["md5"],
        datetime.today(),
        product,
    )

class Database:
    def __init__(self, path):
        self._conn = sqlite3.connect(path)
        self._conn.row_factory = sqlite3.Row
        
        self._prepare()
    
    def _prepare(self):
        c = self._conn
        
        c.execute(_create_table_orders)
        c.execute(_create_table_products)
        c.execute(_create_table_downloads)
        c.execute(_enable_foreign_keys)
        
        c.commit()
    
    def update(self, humble, other_keys):
        keys = humble.get_gamekeys()
        keys += other_keys
        
        c = self._conn
        
        for key in keys:
            order, products = humble.get_order_details(key)
            print("Loading %s" % order["human_name"])
            c.execute(_insert_orders, order2tuple(order))
            
            products = humble.get_products_details(products)
            for product, downloads in products:
                c.execute(_insert_products,
                          product2tuple(product, key))
                
                downloads = humble.get_downloads_details(downloads)
                for download in downloads:
                    c.execute(_insert_downloads,
                              download2tuple(download,
                                             product["machine_name"]))
        
        c.commit()
    
    def purchases(self, order='date', human_name=None):
        c = self._conn
        
        allowed_order = {
            'alpha': "human_name",
            'ralpha': "human_name DESC",
            'date': "datetime(created)",
            'rdate': "datetime(created) DESC",
        }
        order = allowed_order.get(order, allowed_order['date'])
        
        if human_name is None:
            return c.execute(_select_orders % order)
        else:
            if "*" not in human_name:
                human_name = "*" + human_name + "*"
            return c.execute(_select_orders_like % order, (human_name,))
    
    def products(self, order='alpha'):
        c = self._conn
        
        allowed_order = {
            'alpha': "human_name",
            'ralpha': "human_name DESC",
            'date': "datetime(created)",
            'rdate': "datetime(created) DESC",
        }
        order = allowed_order.get(order, allowed_order['alpha'])
        return c.execute(_select_products % order)
    
    def downloads(self, exclude):
        c = self._conn

        if "platforms" in exclude and len(exclude["platforms"]) > 0:
            platforms = ["downloads.platform != '%s'" % x for x in exclude["platforms"]]
            platforms = "AND (" + " AND ".join(platforms) + ")\n"
        else:
            platforms = ""

        if "games" in exclude and len(exclude["games"]) > 0:
            games = ["product_machine_name != '%s'" % x for x in exclude["games"]]
            games = "AND (" + " AND ".join(games) + ")\n"
        else:
            games = ""

        if "files" in exclude and len(exclude["files"]) > 0:
            files = ["downloads.filename NOT GLOB '%s'" % x for x in exclude["files"]]
            files = "AND (" + " AND ".join(files) + ")\n"
        else:
            files = ""

        if "bundles" in exclude and len(exclude["bundles"]) > 0:
            bundles = ["orders.machine_name != '%s'" % x for x in exclude["bundles"]]
            bundles = "AND (" + " AND ".join(bundles) + ")\n"
        else:
            bundles = ""

        filters = platforms + games + files + bundles
        select = _select_downloads % filters

        return c.execute(select)
    
    def find_filename(self, filename):
        c = self._conn.cursor()
        c.execute(_find_filename, (filename,))
        return c.fetchone()

if __name__ == "__main__":
    db = Database("humble.db")
    db._prepare()


